/*
 * This file is part of rasdaman community.
 *
 * Rasdaman community is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Rasdaman community is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with rasdaman community.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright 2003, 2004, 2005, 2006, 2007, 2008, 2009 Peter Baumann /
 rasdaman GmbH.
 *
 * For more information please see <http://www.rasdaman.org>
 * or contact Peter Baumann via <baumann@rasdaman.com>.
 */
/*************************************************************
 *
 * NAME
 *  tilelockbug.cc - C++ example to execute a rasql tilelockbug
 *
 * SYNOPSIS
 *  tilelockbug HOST PORT DATABASE USER PASSWORD
 *
 * DESCRIPTION
 *  Sends a hardwired tilelockbug to rasdaman server HOST listening
 *  at port PORT with database name DATABASE, authenticating
 *  with user USER and password PASSWORD.
 *  The collection used is 'mr'.
 *
 * EXAMPLE
 *  tilelockbug localhost 7001 RASBASE rasadmin rasadmin
 *
 * SEE ALSO
 *  The rasql command line utility allows typing queries
 *  from the command line, it is way more flexible.
 *
 ************************************************************/
#include <iostream>

// Linux needs this for template instantiation
#ifdef EARLY_TEMPLATE
#define __EXECUTABLE__
#ifdef __GNUG__
#include "raslib/template_inst.hh"
#endif
#endif

// this file is generated by the rasdl processor
#include "basictypes.hh"

#define DEBUG_MAIN
#include "debug-clt.hh"


int main( int ac, char** av )
{
    char rasmgrName[255];
    int  rasmgrPort;
    char baseName[255];
    char userName[255];
    char userPass[255];

    if( ac != 6 )
    {
        cout << "Usage: tilelockbug HOST PORT DATABASE USER PASSWORD" << endl;
        return -1;
    }

    strcpy( rasmgrName, av[1] );
    rasmgrPort = strtoul( av[2], NULL, 0);
    strcpy( baseName,   av[3] );
    strcpy( userName,   av[4] );
    strcpy( userPass,   av[5] );

    string createColl    = string("create collection lockbug GreySet");
    string insertDummy   = string("insert into lockbug values marray x in [0:0,0:0] values 0c");
    string queryOID      = string("select oid(m) from lockbug as m");
    string dropColl      = string("drop collection lockbug");

    std::vector<string> queries;
    queries.push_back(createColl);
    queries.push_back(queryOID);
    queries.push_back(insertDummy);
    queries.push_back(queryOID);
    queries.push_back(dropColl);

    r_Database                       database;
    r_Transaction                    transaction;
    r_Set< r_Ref_Any > 			     res_set;

    try
    {
        database.set_servername( rasmgrName, rasmgrPort );
        database.set_useridentification( userName, userPass );

        cout << "Opening database " << baseName
             << " on " << rasmgrName << "... " << flush;

        database.open( baseName );
        cout << "OK" << endl;

        // do some queries ...
        for (int i=0; i < queries.size(); ++i)
        {
			try
			{
				cout << "Starting read-write transaction ... " << flush;
				transaction.begin( r_Transaction::read_write );
				cout << "OK" << endl;

				r_OQL_Query q(queries[i].c_str());
				cout << "executing query: " << q.get_query() << endl;

				if (i % 2 == 0)
					r_oql_execute(q);
				else
					r_oql_execute(q, res_set);

				cout << "Committing transaction ... " << flush;
				transaction.commit();
				cout << "OK" << endl;
			}
			catch( r_Error& errorObj )
			{
				cout << "FAILED" << endl << errorObj.what() << endl;

				cout << "Aborting transaction ... " << flush;
				transaction.abort();
				cout << "OK" << endl;

				cout << "Closing database ... " << flush;
				database.close();
				cout << "OK" << endl;
				return -1;
			}
        }

		cout << "Closing database ... " << flush;
		database.close();
		cout << "OK" << endl;

    }
    catch( r_Error& errorObj )
    {
        cerr << errorObj.what() << endl;
        return -1;
    }

    return 0;
}

/*
 * end of tilelockbug.cc
 */
